/*
 *	@title	Command Line Interface functions, user interface
 *	@author	imperix SA (dev@imperix.ch)
 *	@file	cli_commands.c
 */

#include "DSP28x_Project.h"								// Headers for C28346
#include "Core/macros.h"								// Global defines and macros
#include "Driver/cli.h"									// Command line interface interpreter
#include "../API/transformations.h"						// Coordinate transformations
#include "user.h"										// User-level routines

#include <string.h>
#include <stdlib.h>
#include <stdio.h>



/**
 * Prototype here each user function first!
 */
void DoStart(unsigned int argc, char *argv[]);
void DoStop(unsigned int argc, char *argv[]);
void DoOpMode(unsigned int argc, char *argv[]);
void SetId(unsigned int argc, char *argv[]);
void SetIq(unsigned int argc, char *argv[]);


/**
 * Global variables
 */
extern SpaceVector Ig_dq0_ref;								// Grid current setpoint 
extern USER_STATE opmode;								// Operation mode
 
 
void LoadCLIUserFunctions(void)
{
	/* This function is called by the core at system startup.
	 * the user-defined functions have to be registered
	 * by the command line interpreter before they can be used.
	 *
	 * These functions will be mapped in the /user folder in
	 * the command line mode. Use 'cd user' in order to be in this
	 * folder before using these commands.
	 */
	fs_mkcmd_user("start", DoStart);
	fs_mkcmd_user("stop", DoStop);
	fs_mkcmd_user("opmode", DoOpMode);
	fs_mkcmd_user("setid", SetId);
	fs_mkcmd_user("setiq", SetIq);
	//fs_mkcmd_user("step", step);
	//fs_mkcmd_user("transient", transient);
}


void DoStart(unsigned int argc, char *argv[])
{
	SetOpMode(CLOSEDLOOP);
	printf("\nUser state successfully changed to CONNECTED");
}


void DoStop(unsigned int argc, char *argv[])
{
	SetOpMode(STANDBY);
	printf("\nUser state successfully changed to SHUTDOWN");
}


void DoOpMode(unsigned int argc, char *argv[])
{
	int newmode;
	if (argc > 1)
	{
		// There is an argument passed to the command
		if (sscanf(argv[1], "%u", &newmode))
		{
			// A number has been sent as argument
			// The user wants to change the opmode
			SetOpMode((USER_STATE) newmode);
		}
		else
		{
			printf("\nUnknown argument.");
			printf("\nSet the opmode : opmode 3");
			printf("\n                 3 is the number (ID) of the new mode");
			printf("\n                 See user.h for links between mode name and ID.");
			printf("\nGet current opmode : opmode");
		}

	}
	printf("\nMode is %u", opmode);
}


void SetId(unsigned int argc, char *argv[])
{
	if(argc == 2)
	{
		Ig_dq0_ref.real = atof(argv[1]);
		printf("\nThe Id reference current value successfully changed");
	}

}


void SetIq(unsigned int argc, char *argv[])
{
	if(argc == 2)
	{
		Ig_dq0_ref.imaginary = atof(argv[1]);
		printf("\nThe Iq reference current value successfully changed");
	}

}


// EXAMPLES OF POSSIBLE USE OF THE DATALOGGING:
/*
void step(unsigned int argc, char *argv[])
{
	if (*argv[1] == '?'){
		printf("\nSet a step function combined with the datalogging.");
		printf("\nUsage : step fct_number tstart target_address delta_value");
		printf("\nExample : step 1 256 0x00C020 8.0");
		return;
	}
	if ((*argv[1] >= '0') && (*argv[1] < ('0' + 4))){
		int fct_number = (*argv[1]-'0');

		ConfigTransient(&(mylog.step[fct_number]), atoi(argv[2]), (float32*)strtoul(argv[3],NULL,16), atof(argv[4]));
	}
	else{
		printf("\nIncorrect step function index. Must be 0-3.");
		return;
	}
}


void transient(unsigned int argc, char *argv[])
{
	if (*argv[1] == '?'){
		printf("\nLaunches the datalogging along with the registered steps");
		printf("\nUsage : no arguments");
		return;
	}
	else{
		LaunchTransient(&mylog);
	}
}
*/
