/*
 *  @title    Closed-loop control of a grid-connected single-phase inverter
 *	@author   imperix SA (dev@imperix.ch)
 *	@file     user.c
 */

#include "Driver/peripherals.h"								// Available DSP peripherals
#include "Core/interrupts.h"								// Interrupt-related routines
#include "user.h"											// This file's header.
#include <stdio.h>											// Standard IO, typ. used for printf()
#include <math.h>											// Mathematical functions

#include "../API/transformations.h"							// Coordinate transformations
#include "../API/controllers.h"								// Discrete-time controllers
#include "../API/PLLs.h"									// PLLs


#define THREELEVEL											// Comment this line for two-level modulation

/**
 * Global variables
 * Please pay attention not to define variables that are unnecessarily global !
 */
USER_STATE opmode, next_opmode = STANDBY;					// Defines the operation mode of the converter
Uint16 InterruptCounter = 0;								// Is used to control the time when no PLL is used

// Variables for synchronization of converter with phase
SpaceVector Ug_ABG, Ug_dq0;									// Static and rotating reference frame grid voltage
SpaceVector Ig_ABG, Ig_dq0; 								// Static and rotating reference frame grid current
SpaceVector Ig_dq0_ref;										// Rotating reference frame grid current setpoint
SpaceVector Eg_ABG, Eg_dq0; 								// Static and rotating reference frame converter voltage

SOGIPLL1Parameters SOGIPLL;									// Pseudo-object containing the PLL parameters (Synchronisation with the grid)
FAEParameters FAE;											// Pseudo-object containing the FAE parameters (Estimate the current grid on q-axis)
PIDController Id_reg, Iq_reg;								// Line current controller parameters for d and q axis

float Udc;													// Intermediate bus voltage (actual)
float Ug_measured;											// Raw grid voltage measurement
float theta = 0.0;											// Phase angle of the grid voltage
float duty_a, duty_b = 0.0;									// Duty cycles for the two half bridges
float moddepth = 0.0;										// Modulation depth (is used in open-loop modulation with a passive load)


/**
 * The user should put here his/her initialization routines.
 * This function is called only once at startup.
 */
USER_SAFE UserInit(void)
{
	// Configure a 20 kHz switching frequency on clock source #0:
	SetFreqGenPeriod(0, (int)(SAMPLING_PERIOD/FPGA_CLK_PERIOD));

	// Map PWM channel #0 to clksource #1 with a triangular carrier and a 400 ns deadtime:
	ConfigPWMChannel(1, 0, TRIANGLE, (int)(400e-9/FPGA_CLK_PERIOD));
#ifdef THREELEVEL
	ConfigPWMChannel(2, 0, TRIANGLE, (int)(400e-9/FPGA_CLK_PERIOD));
#else
	ConfigPWMChannel(2, 0, INVTRIANGLE, (int)(400e-9/FPGA_CLK_PERIOD));
#endif

	// Configure PWM channels #0 and #1 with a phase of zero with respect to their FG:
	SetPWMPhase(1, 0.0);
	SetPWMPhase(2, 0.0);

	// Analog input #0 to #2 with corrective gain and offset:
	SetADCAdjustments(0, 6.10e-3/4, 0.0);					// Nominal sensitivity and x4 gain (Ig)
	SetADCAdjustments(1, 57.4e-3/8, 0.0);					// Nominal sensitivity and x8 gain (Ug)
	SetADCAdjustments(3, 57.4e-3/8, 0.0);					// Nominal sensitivity and x8 gain (Udc)

	// Configure UsrInt1 on the PWM channel #0:
	RegisterExt1Interrupt(&UserInterrupt1, 0, 0.0, 0);
	
	// Configure the sampling in the middle of the period of the FreqGen #0:
	ConfigSampling(0, 0.5);

	// Configure and initialize the controllers:
	#define Kp 10.0
	#define Ki 0.2
	#define Kd 0.0
	ConfigPIDController(&Id_reg, Kp, Ki, 0, 30.0, -30.0, SAMPLING_PERIOD, 10);
	ConfigPIDController(&Iq_reg, Kp, Ki, 0, 30.0, -30.0, SAMPLING_PERIOD, 10);

	// Configure and initialize the PLL and the FAE:
	ConfigSOGIPLL1(&SOGIPLL, 10.0, 0.5, 0.1, OMEGA, SAMPLING_PERIOD);
	ConfigFAE(&FAE, RGRID, LGRID, SAMPLING_PERIOD);

	// Initialize all setpoints:
	Ig_dq0_ref.real = 0.0;
	Ig_dq0_ref.imaginary = 0.0;

	// Prepare for the first execution:
	opmode = STANDBY;										// Initialize the application to its STANDBY state
	return SAFE;											// Return 0 if no error
}


/**
 * First user-level interrupt service routine.
 * This routine is synchronous to the PWM #0 period
 */
USER_SAFE UserInterrupt1(void)
{
	// Measure all the necessary quantities and emulate the missing one:
	Ug_measured = GetADC(1);								// Grid voltage
	Udc = GetADC(3);										// DC bus voltage
	Ig_ABG.real = GetADC(0);								// Grid current
	Ig_ABG.imaginary = RunFAE(&FAE, Eg_ABG.imaginary - Ug_ABG.imaginary);

	// Run the SOGI-based PLL to synchronize with the grid:
	theta = RunSOGIPLL1(&SOGIPLL,&Ug_ABG,Ug_measured);

	// Switch depending on the operation mode (state machine)
	switch (opmode){
	   	case STANDBY:		// Stay in this mode as long as the execution mode is not manually changed:
	   						next_opmode = STANDBY;
							break;

	   	case CLOSEDLOOP:	// Compute the necessary coordinate transformations:
							ABG2DQ0(&Ug_dq0,&Ug_ABG,theta);	// Grid voltage
							ABG2DQ0(&Ig_dq0,&Ig_ABG,theta);	// Grid current

							// Run a Multivariable DQ current controller:
	   						Eg_dq0.real = Ug_dq0.real + RunPIController(&Id_reg,Ig_dq0_ref.real-Ig_dq0.real) - OMEGA*LGRID*Ig_dq0.imaginary;
	   						Eg_dq0.imaginary = Ug_dq0.imaginary + RunPIController(&Iq_reg,Ig_dq0_ref.imaginary-Ig_dq0.imaginary) + OMEGA*LGRID*Ig_dq0.real;
	   						Eg_dq0.offset = 0.0;

	   						// Compute the inverse coordinate transformations:
	   						DQ02ABG(&Eg_ABG,&Eg_dq0,theta);	// Convert the converter voltage to the stationary reference frame

	   						// Compute the duty-cycles:
	   						duty_a = 0.5 + 0.5* (Eg_ABG.real/Udc);
	   						duty_b = 0.5 - 0.5* (Eg_ABG.real/Udc);

	   						// Stay in this mode as long as the execution mode is not manually changed:
	   						next_opmode = CLOSEDLOOP;
	   						break;

	   	case PASSIVELOAD:	// Override the PLL (no grid is present):
	   						if (InterruptCounter++==(int)(SIGNAL_PERIOD/SAMPLING_PERIOD)) InterruptCounter=0;
	   						theta = OMEGA*SAMPLING_PERIOD*InterruptCounter;
							if (theta > PI) {SetGPObit(1);} else {ClearGPObit(1);}

	   						// Execute a strictly open-loop modulation:
	   						duty_a = 0.5 + moddepth * sin(theta);
	   						duty_b = 0.5 - moddepth * sin(theta);

	   						// Stay in this mode as long as the execution mode is not manually changed:
	   						next_opmode = PASSIVELOAD;
	   						break;
		}

	// Display some quantities for debug purposes:
	SetDACVoltage(0,Ig_ABG.real/IG_NOMINAL);
	SetDACVoltage(1,Ig_ABG.imaginary/IG_NOMINAL);
	SetDACVoltage(2,Ig_dq0.real/IG_NOMINAL);
	SetDACVoltage(3,Ig_dq0.imaginary/IG_NOMINAL);

	// Set the duty-cycles:
	SetPWMDutyCycle(1,duty_a);
	SetPWMDutyCycle(2,duty_b);

	// Update the modulators:
	UpdatePWMData();

	// Prepare for the next function call:
	SetOpMode(next_opmode);									// Switch to the next state
	return SAFE;											// Return a SAFE status (or UNSAFE otherwise)
}


/**
 * First user-level interrupt service routine.
 * This routine is used with a relatively long interrupt period and is only
 * responsible for executing the MPPT "pertub and assess" algorithm
 */
USER_SAFE UserInterrupt2(void)
{
	return SAFE;											// Nothing is done here !
}


/*
 * This routine aims to handle critical errors by requesting an appropriate answer at the user level.
 * Typically, the application should be blocked and the core will wait for manual clearing of the error.
 */
void UserError(ERROR_SOURCE source)
{
	printf("\nUserError() has been triggered!");			// Only a warning message is displayed here
}


/*
 * This routine is responsible for changing the mode of operation, when possible and applicable
 */
void SetOpMode(USER_STATE newmode)
{
	// Switch depending on the desired execution mode:
	switch (newmode){
	   	case STANDBY:		DeactivatePWMChannel(1);		// Disable Half bridge 0
	   						DeactivatePWMChannel(2);		// Disable Half bridge 1

	   						// Reset the setpoints:
	   						Ig_dq0_ref.real = 0.0;
	   						Ig_dq0_ref.imaginary = 0.0;
							break;

	   	case CLOSEDLOOP:	ActivatePWMChannel(1);
	   			   			ActivatePWMChannel(2);
	   		   				break;

	   	case PASSIVELOAD:	if (opmode == CLOSEDLOOP){
	   							printf("\nThe direct transition from CLOSEDLOOP is not allowed.");
	   							printf("\nPlease switch to STANDBY and make sure no grid is present first.");
	   							return;						// Skip the opmode = newmode part.
	   						}
	   						else{
								ActivatePWMChannel(1);
								ActivatePWMChannel(2);
								break;
	   						}
		}

	// Actually change the execution mode:
	if (1){													// All transitions are allowed here
		opmode = newmode;
	}
}
